exports = module.exports = (function () {
    'use strict'

    const storage = require('./storage')

    function timeFormat(time, format = 'YYYY-MM-DD HH:mm:ss') {
        if (!time) throw new Error('Time must be provided!')

        const t = new Date(time) // 2019-11-09T06:27:57.040Z
        const tf = (i) => { return (i < 10 ? '0' : '') + i }

        return format.replace(/YYYY|MM|DD|HH|mm|ss/g, (key) => {
            switch (key) {
                case 'YYYY':
                    return tf(t.getFullYear())
                case 'MM':
                    return tf(t.getMonth() + 1)
                case 'DD':
                    return tf(t.getDate())
                case 'HH':
                    return tf(t.getHours())
                case 'mm':
                    return tf(t.getMinutes())
                case 'ss':
                    return tf(t.getSeconds())
            }
        })
    }

    function getNow(format = 'YYYY-MM-DD HH:mm:ss') {
        return timeFormat(new Date(), format)
    }

    function checkStrong(val) {
        if (!val) throw new Error('String must be provided!')

        let modes = 0

        if (val.length < 1) return modes
        if (/\d/.test(val)) modes++ // number
        if (/[a-z]/.test(val)) modes++ // lower
        if (/[A-Z]/.test(val)) modes++ // upper  
        if (/\W/.test(val)) modes++ // special

        switch (modes) {
            case 1:
                return 1
            case 2:
                return 2
            case 3:
            case 4:
                return val.length < 12 ? 3 : 4
        }
        return modes
    }


    return {
        storage,
        timeFormat,
        getNow,
        checkStrong
    }
})()