exports = module.exports = (function () {
    'use strict'

    let store = {
        local: { storage: window.localStorage },
        session: { storage: window.sessionStorage },
        cookie: {
            storage: window.document.cookie,
            set(fieldKey, fieldVal, exdays) {
                /* set expire time */
                const exdate = new Date()
                exdate.setTime(exdate.getTime() + 24 * 60 * 60 * 1000 * exdays)

                /* set cookie */
                this.storage =
                    fieldKey +
                    "=" +
                    fieldVal +
                    ";path=/;expires=" +
                    exdate.toGMTString()
            },
            get() {
                let result = []
                if (this.storage.length > 0) {
                    /* split */
                    const arr = this.storage.split(";")

                    for (let i = 0; i < arr.length; i++) {
                        /* split again */
                        const arr2 = arr[i].split("=")
                        result.push(arr2[1])
                    }
                }
                return result
            },
            clear() {
                this.set("", "", -1)
            }
        }
    }

    function serialize(val) {
        return JSON.stringify(val)
    }

    function deserialize(val) {
        if (typeof val !== 'string') {
            return undefined
        }
        return JSON.parse(val)
    }

    const api = {
        set(key, val) {
            this.storage.setItem(key, serialize(val))
        },

        get(key) {
            let val = deserialize(this.storage.getItem(key))
            if (val === undefined) {
                return undefined
            } else {
                return val
            }
        },

        has(key) {
            return this.get(key) !== undefined
        },

        remove(key) {
            this.storage.removeItem(key)
        },

        clear() {
            this.storage.clear()
        },

        getAll() {
            let ret = {}
            this.forEach((key, val) => {
                ret[key] = val
            })
            return ret
        },

        forEach(callback) {
            for (let i = 0; i < this.storage.length; i++) {
                let key = this.storage.key(i)
                callback(key, this.get(key))
            }
        }
    }

    /* 拼接 */
    Object.assign(store.local.storage, api)
    Object.assign(store.session.storage, api)

    return store
})()